import java.util.Vector;
import java.sql.Types.*;
import java.math.BigDecimal;

public class Encap implements Encapsulation,Cloneable
{
 public final static int INT = java.sql.Types.INTEGER;
 public final static int STR = java.sql.Types.VARCHAR; 
 public final static int FLT = java.sql.Types.FLOAT;
 public final static int DBL = java.sql.Types.DOUBLE;
 public final static int LNG = java.sql.Types.BIGINT;
 public final static int VAR = 99;
 public final static int DAT = java.sql.Types.DATE;
 public final static int TIM = java.sql.Types.TIME;
 public final static int DEC = java.sql.Types.DECIMAL;

 private StringBuffer encapCond = null; // the database condition set
 private StringBuffer encapTable= null; // the database table name 
 private StringBuffer encapVar  = null; // the database variable (ie, @user) 
 // the value of the encap data
 private Vector encapValues;
 private Vector encapNames;
 private Vector encapTypes;

 public Encap() { init(); }
 public Encap(String table) 
 { 
  init(); 
  setTable(table); 
 }
 public Encap(String table,String name,String value)
 {
  init();
  setTable(table);
  setNameVal(name,value);
 }
 public Encap(String table,String name,int value)
 {
  init();
  setTable(table);
  setNameVal(name,value);
 }
 public Encap(String table,String name,long value)
 {
  init();
  setTable(table);
  setNameVal(name,value);
 }
 public Encap(String table,String name,float value)
 {
  init();
  setTable(table);
  setNameVal(name,value);
 }
 public Encap(String table,String name,double value)
 {
  init();
  setTable(table);
  setNameVal(name,value);
 }
 public Encap(String table,String name,double value,boolean dec)
 {
  init();
  setTable(table);
  setNameVal(name,value,dec);
 }
 public Encap(String table,String name,BigDecimal value)
 {
  init();
  setTable(table);
  setNameVal(name,value);
 }
 public void init()
 {
  encapValues = new Vector();
  encapNames = new Vector();
  encapTypes = new Vector();
  encapVar   = new StringBuffer();
  encapTable = new StringBuffer();
  encapCond  = new StringBuffer();
 }

 public void clear()
 {
  if(!encapValues.isEmpty())
   encapValues.removeAllElements();
  if(!encapNames.isEmpty())
   encapNames.removeAllElements();
  if(!encapTypes.isEmpty())
   encapTypes.removeAllElements();
  if(encapCond.length()>0)
   encapCond.delete(0,encapCond.length());
  if(encapTable.length()>0)
   encapTable.delete(0,encapTable.length());
  if(encapVar.length()>0)
   encapVar.delete(0,encapVar.length());
 }

 public void removeNameVal(int pos)
 { 
  removeVal(pos); 
  removeName(pos);
 }

 public void removeVal(int pos)
 {
  encapValues.removeElementAt(pos);
  encapTypes.removeElementAt(pos);
 }

 public void removeName(int pos)
 { encapNames.removeElementAt(pos); }

 public void setNameVal(String name, String value)
 { 
  setName(name); 
  setValue(value); 
 }

 public void setNameVal(String name, int value)
 { 
  setName(name); 
  setValue(value); 
 }

 public void setNameVal(String name, float value)
 { 
  setName(name); 
  setValue(value); 
 }

 public void setNameVal(String name, double value)
 { 
  setName(name); 
  setValue(value); 
 }

 public void setNameVal(String name, long value)
 {
  setName(name);
  setValue(value);
 }

 public void setNameVal(String name, BigDecimal value)
 {
  setName(name);
  setValue(value);
 }

 public void setNameVal(String name, double value, boolean dec)
 {
  setName(name);
  setDecValue(value);
 }

 public void setTable(String table)
 { 
  if(encapTable.length()>0) encapTable.delete(0,encapTable.length());
  encapTable.append(table); 
 }

 public void setName(String name)
 { encapNames.add(new String(name)); }

 public void setCondition(String cond)
 { encapCond.append(cond); } 

 public void setVariable(String name)
 { encapVar.append(name); }

 public void setValue(BigDecimal value)
 { encapValues.add(value); encapTypes.add(new Integer(DEC)); }

 public void setValue(int value)
 { encapValues.add(new Integer(value)); encapTypes.add(new Integer(INT)); }

 public void setValue(float value)
 { encapValues.add(new Float(value)); encapTypes.add(new Float(FLT));  }

 public void setValue(double value)
 { encapValues.add(new Double(value)); encapTypes.add(new Integer(DBL)); }

 public void setDecValue(double value)
 { encapValues.add(new BigDecimal(value)); encapTypes.add(new Integer(DEC)); }

 public void setValue(long value)
 { encapValues.add(new Long(value)); encapTypes.add(new Integer(LNG)); }

 public void setValue(String value)
 { 
  if(value == null)
  {
   encapValues.add(new String("none"));
   encapTypes.add(new Integer(STR));
  }
  else if(value.length()==0 || value.compareToIgnoreCase(" ")==0)
  {
   encapValues.add(new String("none"));
   encapTypes.add(new Integer(STR));
  }
  else
  {
   encapValues.add(new String(value)); 
   if(value.charAt(0)=='@')
    encapTypes.add(new Integer(VAR));
   else if(value.startsWith("PASSWORD"))
    encapTypes.add(new Integer(VAR));
   else if(value.compareToIgnoreCase("CURRENT_DATE")==0)
    encapTypes.add(new Integer(DAT));
	 else if(value.compareToIgnoreCase("CURRENT_TIME")==0)
		encapTypes.add(new Integer(TIM));
   else
    encapTypes.add(new Integer(STR)); 
  }
 }

 public int getPosition(String name)
 { return encapNames.indexOf(name); }

 public int getSize()
 { return encapNames.size(); }

 public String getCondition()
 { 
  if(encapCond.length()==0)
   return null; 
  return encapCond.toString(); 
 }
 public String getVariable()
 { 
  if(encapVar.length()==0)
   return null;
  return encapVar.toString(); 
 }
 public int getValueSize()
 { return encapValues.size(); }

 public int getType(int pos)
 { return ((Integer)encapTypes.elementAt(pos)).intValue(); }

 public String getValue(int pos)
 { 
  switch(getType(pos))
  {
   case INT : return ((Integer)encapValues.elementAt(pos)).toString();
   case DAT :
   case VAR :
	 case TIM :
   case STR : return encapValues.elementAt(pos).toString(); 
   case FLT : return ((Float)encapValues.elementAt(pos)).toString();
   case DBL : return ((Double)encapValues.elementAt(pos)).toString(); 
   case DEC : return ((BigDecimal)encapValues.elementAt(pos)).toString();
   case LNG : return ((Long)encapValues.elementAt(pos)).toString();
   default  : 
  }
  return null;
 }

 public int getIntValue(int pos)
 { return ((Integer)encapValues.elementAt(pos)).intValue(); } 

 public float getFloatValue(int pos)
 { return ((Float)encapValues.elementAt(pos)).floatValue(); }

 public double getDoubleValue(int pos)
 { return ((Double)encapValues.elementAt(pos)).doubleValue(); }

 public long getLongValue(int pos)
 { return ((Long)encapValues.elementAt(pos)).longValue(); }

 public int getIntLongValue(int pos)
 { return ((Long)encapValues.elementAt(pos)).intValue(); }

 public double getDecDoubleValue(int pos)
 { return ((BigDecimal)encapValues.elementAt(pos)).doubleValue(); }

 public BigDecimal getDecValue(int pos)
 { return (BigDecimal)encapValues.elementAt(pos); }

 public String getStringValue(int pos)
 { return (String)encapValues.elementAt(pos); }

 public String getTable()
 { 
  if(encapTable.length()==0)
   return null;
  return encapTable.toString(); 
 }
 public String getName(int pos)
 { return (String)encapNames.elementAt(pos); }

 public String extractAll(String name)
 {
  int p = getPosition(name);
  if(p<0) return null;
  return getValue(p);
 }
 public double extractDouble(String name)
 {
  int p = getPosition(name);
  if(p<0) return 0;
  if(((Integer)encapTypes.elementAt(p)).intValue() == DEC)
    return getDecDoubleValue(p);
  return getDoubleValue(p);
 }
 public BigDecimal extractBigDecimal(String name)
 {
  int p = getPosition(name);
	if(p<0) return null;
	return getDecValue(p);
 }
 public float extractFloat(String name)
 {
  int p = getPosition(name);
  if(p<0) return 0;
  return getFloatValue(p);
 }
 public String extractString(String name,String value)
 {
  int p = getPosition(name);
  if(p<0) return null;
  if(value == null) 
   return getStringValue(p);
  while((p<getSize())&&(p>0))
  {
   if(getStringValue(p).compareToIgnoreCase(value)==0)
    return getStringValue(p);
   p = encapNames.indexOf(name,p+1); 
  }
  return null;
 }
 public int extractInteger(String name,int value)
 {
  int p = getPosition(name);
  if(p<0) return 0;
  if(value<=0) return getIntValue(p);
  while((p<getSize())&&(p>0))
  {
   if(getIntValue(p)==value)
    return getIntValue(p);
   p = encapNames.indexOf(name,p+1);
  }
  return 0;
 }

 public String extractName(String value)
 {
  int p = encapValues.indexOf(value);
  if(p<0) return null;
  return getName(p);
 }

 public void replaceValue(int pos,Object obj)
 {
  int valType = getType(pos);
  switch(valType)
  {
	 case TIM :
	 case DAT :
   case VAR :
   case STR : encapValues.setElementAt((String)obj,pos); break;
   case INT : encapValues.setElementAt((Integer)obj,pos); break;
   case FLT : encapValues.setElementAt((Float)obj,pos); break;
   case DBL : encapValues.setElementAt((Double)obj,pos); break;
   case LNG : encapValues.setElementAt((Long)obj,pos); break; 
   case DEC : encapValues.setElementAt((BigDecimal)obj,pos); break;
  }
  return;
 }

 protected Object clone()
 {
  try
  {
   Encap enCopy = (Encap)super.clone(); 
   enCopy.init();
   enCopy.encapNames = (Vector)encapNames.clone();
   enCopy.encapValues= (Vector)encapValues.clone();
   enCopy.encapTypes = (Vector)encapTypes.clone();
   if(getTable()!= null)
    enCopy.setTable(getTable());
   if(getCondition()!= null)
    enCopy.setCondition(getCondition());
   if(getVariable()!=null)
    enCopy.setVariable(getVariable());
   return enCopy;
  }
  catch(CloneNotSupportedException e)
  {
   e.printStackTrace(System.out);
  }
  return this;
 }
}
