import java.math.BigDecimal;
import java.util.Vector;
import java.sql.*;
import javax.sql.*;
import java.io.*;

public class FetchData
{
 public final static int SUMMAR = 1;
 public final static int SUMMAP = 2;
 public final static int SUMPAR = 3;
 public final static int SUMPAP = 4;

 private int action = 0;
 private DBConnect dbcon = null; 
 private StringBuffer error = new StringBuffer();
 private BigDecimal bdTotal = null;
 private String currTotal = null;
 public MultipleCurrency mc = new MultipleCurrency();

 public FetchData(int act)
 { action = act; } 

 public Vector fetch(int act)
 {
  if(act<=0) act = action;
  if(error.length()>0) error.delete(0,error.length());

  switch(act)
  {
   case SUMMAR : return fetchAcctRec();
   case SUMMAP : return fetchAcctPay();
   case SUMPAR : return fetchPaidAR();
   case SUMPAP : return fetchPaidAP();
  }
  error.append("No service available for that chosen action\n");
  return null;
 }

 public String getError()
 {
  return error.toString();
 }

 public String getCurrTotal()
 {
  if(currTotal == null) return new String("-0-");
  return currTotal;
 }

/**
 * fetchAcctRec is to fetch the outstanding account receivable
 * @param none
 * @return Vector of all the outstanding A/R
 */
 public Vector fetchAcctRec()
 {
  dbcon = new DBConnect();
  StringBuffer msg = new StringBuffer(48);
  Vector v = null;
  try
  {
   if(!dbcon.makeConnection("LKB"))
   {
    error.append("Failure to make connection to database\n");
    throw new SQLException("Failed to make connection");
   }
   msg.append("LEFT JOIN customer ON (accountReceivable.acctRecFromCustKey=");
   msg.append("customer.customerKey) LEFT JOIN transRecord ON ");
   msg.append("(accountReceivable.acctRecFromRefKey=transRecord.transRefKey) ");
   msg.append("WHERE accountReceivable.acctRecPaidFull=\"N\" ");
   msg.append("ORDER BY transRecord.transRefKey;");
   v = dbcon.QueryMulti("accountReceivable","*",msg.toString()); 
   if(v == null)
   {
    error.append("No available data for the requested information\n");
    error.append("Either there are no more active account or\n");
    error.append("the database is still empty\n");
    return null;
   }
   mc.clearVariable();
   currTotal = null;
   v = rearrange(fetchTransVol(v),"acctRecBalance","acctRecCurr");
  }
  catch(Exception ex)
  {
   System.out.println("Exception: "+ex.getMessage());
   ex.printStackTrace(System.out);
   error.append(ex.getMessage());
   if(v!=null && !v.isEmpty()) 
   {
    v.removeAllElements();
    v = null;
   }
  }
  finally
  {
   dbcon.close();
   msg.delete(0,msg.length());
  }
  return v;
 }

 private Vector fetchTransVol(Vector v) throws SQLException
 {
  Vector v2 = new Vector();
  Encap e = new Encap();
  Encap e2= new Encap();
  int v1size= v.size();
  int v2size= 0;
  int index = 1;
  for(int i=0;i<v1size;i++)
  {
   e = (Encap)(((Encap)v.get(i)).clone()); 
   v2 = dbcon.QueryMulti("transVol","*",
        "WHERE transVolRefKey="+e.extractInteger("transRefKey",0)+";");
   if(v2!=null)
   {
    v2size = v2.size();
    e.setNameVal("transVolSize",v2size);
    do
    {
     e2 = (Encap)v2.remove(0);
     e.setNameVal("transVolVolume"+index,
      e2.extractInteger("transVolVolume",0));
     e.setNameVal("transVolStowage"+index,
      e2.extractString("transVolStowage",null));
     index++;
     e2.clear();
    }while(!v2.isEmpty());
    index = 1;
   }
   v.setElementAt(e.clone(),i);
   e.clear();
  }
  return v;
 }

/**
 * rearrange's method is to rearrange result from the query [for outstanding
 * and paid] accounts.
 * @param Vector v contains the data to be rearrange
 *        String amount contains the amount 
 *        String curr contains the amount currency
 * @return Vector 
 */
 private Vector rearrange(Vector v,String amount,String curr) 
	throws SQLException
 {
  Vector v2 = new Vector();
  Encap en = null; 
  Encap en2= null;
  int pos = 0;
  int key = 0;
  int volSize = 0;
  StringBuffer ty  = new StringBuffer();
  StringBuffer str = new StringBuffer();
  StringBuffer lode= new StringBuffer();
  StringBuffer shgo= new StringBuffer();
  StringBuffer name= new StringBuffer();
  boolean zeroFlag = false;
  while(v.size()>0) 
  {
   if(en!=null) en.clear();
   if(en2!=null) en2.clear();
   if(ty.length()>0) ty.delete(0,ty.length());
   if(name.length()>0) name.delete(0,name.length());
   if(str.length()>0) str.delete(0,str.length());
   if(lode.length()>0) lode.delete(0,lode.length());
   if(shgo.length()>0) shgo.delete(0,shgo.length());
   en = (Encap)v.remove(0);
   switch(action)
   {
    case SUMMAR:
    case SUMPAR:
     if(en.extractAll("acctRecAmt").equalsIgnoreCase("0.00"))
      zeroFlag = true;
     break;
    case SUMMAP:
    case SUMPAP:
     if(en.extractAll("acctPayAmt").equalsIgnoreCase("0.00"))
      zeroFlag = true;
     break;
    default:
   }
   
   if(!zeroFlag)
   {
    key = en.getIntValue(en.getPosition("transRefKey"));
    pos = en.getPosition("transType");
    ty.append(en.getValue(pos));
    if(ty.toString().equalsIgnoreCase("IMP"))
    {
     str.append("WHERE impFromRefKey=\""+key+"\";"); 
     en2 = dbcon.QuerySingle("impInfoRecord","*",str.toString());
     if(en2 == null)
      throw new SQLException("No available Import Information\n");
     lode.append(en.getStringValue(en.getPosition("transLoadingPort")));
     shgo.append(en2.getValue(en2.getPosition("impGateOut")));
    }
    else // must be export
    {
     str.append("WHERE expFromRefKey=\""+key+"\";");
     en2 = dbcon.QuerySingle("expInfoRecord","*",str.toString());
     if(en2 == null)
      throw new SQLException("No available Export Information\n");
     lode.append(en2.getStringValue(en2.getPosition("expDestName")));
     shgo.append(en2.getStringValue(en2.getPosition("expStuffingDate")));
    } 
    if((action==SUMMAR)||(action==SUMPAR))
     name.append(en.getStringValue(en.getPosition("customerName")));
    else
     name.append(en.getStringValue(en.getPosition("truckingName")));
    mc.sumAR(en.extractAll(curr),en.extractAll(amount));
    currTotal = en.getValue(en.getPosition(curr));
    SummaryData smData = new SummaryData();
    smData.setRef(key);
    smData.setIE(ty.toString());
    smData.setNm(name.toString());
    str.delete(0,str.length());
    volSize = en.extractInteger("transVolSize",0);
    for(int i=1;i<=volSize;i++)
    {
     if(i>1) str.append(",");
     str.append(en.extractAll("transVolVolume"+i)+
      "x"+en.extractAll("transVolStowage"+i));
    }
    smData.setVolStw(str.toString());
    smData.setLoDest(lode.toString());
    smData.setDoLoc(en.extractString("transDoorLocation",null));
    smData.setCurrency(en.extractString(curr,null));
    smData.setTotal(Module7.formNumDelimiter(en.extractAll(amount)));
    smData.setInvDate(
     Module7.formNDate(en.extractString("transInvoiceDT",null)));
    smData.setShipGO(Module7.formNDate(shgo.toString())); 
    v2.add(smData);
    str.delete(0,str.length());
   }
   else // it must be true then reset
    zeroFlag = false;
  } // end of while-loop
  return v2;
 } // end of rearrangeAR's method

/**
 * fetchAcctPay's method is to fetch the outstanding AP
 * @param none
 * @return Vector of outstanding AP
 */
 public Vector fetchAcctPay()
 {
  dbcon = new DBConnect();
  StringBuffer msg = new StringBuffer();
  Vector v = null;
  try
  {
   if(!dbcon.makeConnection("LKB"))
    throw new SQLException("Failed to make connection to database\n");
   msg.append("LEFT JOIN truckingCompany ON ");
   msg.append("(accountPayable.acctPayToTruckCoKey=");
   msg.append("truckingCompany.truckingCompanyKey) LEFT JOIN transRecord ON ");
   msg.append("(accountPayable.acctPayToRefKey=transRecord.transRefKey) ");
   msg.append("WHERE accountPayable.acctPayPaidFull=\"N\" ");
   msg.append("ORDER BY transRecord.transRefKey;");
   v = dbcon.QueryMulti("accountPayable","*",msg.toString()); 
   if(v == null)
   {
    error.append("No available data for the requested information\n");
    error.append("Either there are no more of active account or\n");
    error.append("the database is still empty\n");
    return null;
   }
   mc.clearVariable();
   currTotal = null;
   v = rearrange(fetchTransVol(v),"acctPayBalance","acctPayCurr");
  }
  catch(Exception ex)
  {
   System.out.println("Exception: "+ex.getMessage());
   ex.printStackTrace(System.out);
   error.append(ex.getMessage());
   if(v!=null && !v.isEmpty())
   {
    v.removeAllElements();
    v = null;
   }
  }
  finally
  {
   dbcon.close();
   msg.delete(0,msg.length());
  }
  return v;
 }

/**
 * fetchPaidAR's method is to fetch paid AR
 * @param none
 * @return Vector of paid AR
 */
 public Vector fetchPaidAR()
 {
  dbcon = new DBConnect();
  StringBuffer msg = new StringBuffer();
  Vector v = null;
  try
  {
   if(!dbcon.makeConnection("LKB"))
   {
    error.append("Failure to make connection to database\n");
    throw new SQLException("Failed to make connection");
   }
   msg.append("LEFT JOIN customer ON (accountReceivable.acctRecFromCustKey=");
   msg.append("customer.customerKey) LEFT JOIN transRecord ON ");
   msg.append("(accountReceivable.acctRecFromRefKey=transRecord.transRefKey) ");
   msg.append("WHERE accountReceivable.acctRecPaidFull=\"Y\" ");
   msg.append("ORDER BY transRecord.transRefKey;");
   v = dbcon.QueryMulti("accountReceivable","*",msg.toString()); 
   if(v == null)
   {
    error.append("No available data for the requested information\n");
    error.append("Either there are no more of active account or\n");
    error.append("the database is still empty\n");
    return null;
   }
   currTotal = null;
   mc.clearVariable();
   v = rearrange(fetchTransVol(v),"acctRecAmt","acctRecCurr");
  }
  catch(Exception ex)
  {
   System.out.println("Exception: "+ex.getMessage());
   ex.printStackTrace(System.out);
   error.append(ex.getMessage());
   if(v!=null && !v.isEmpty())
   {
    v.removeAllElements();
    v = null;
   }
  }
  finally
  {
   dbcon.close();
   msg.delete(0,msg.length());
  }
  return v;
 }

/**
 * fetchPaidAP's method is to fetch the paid account of AP
 * @param none
 * @return Vector of paid AP
 */
 public Vector fetchPaidAP()
 {
  dbcon = new DBConnect();
  StringBuffer msg = new StringBuffer();
  Encap en = null;
  Vector v = null;
  try
  {
   if(!dbcon.makeConnection("LKB"))
    throw new SQLException("Failed to make connection");
   msg.append("LEFT JOIN truckingCompany ON (accountPayable.");
   msg.append("acctPayToTruckCoKey=truckingCompany.truckingCompanyKey) "); 
   msg.append("LEFT JOIN transRecord ON ");
   msg.append("(accountPayable.acctPayToRefKey=transRecord.transRefKey) ");
   msg.append("WHERE accountPayable.acctPayPaidFull=\"Y\" ");
   msg.append("ORDER BY transRecord.transRefKey;");
   v = dbcon.QueryMulti("accountPayable","*",msg.toString()); 
   if(v == null)
   {
    error.append("No available data for the requested information\n");
    error.append("Either there are no more of active account or\n");
    error.append("the database is still empty\n");
    return null; 
   }
   currTotal = null;
   mc.clearVariable();
   v = rearrange(fetchTransVol(v),"acctPayAmt","acctPayCurr");
  }
  catch(Exception ex)
  {
   System.out.println("Exception: "+ex.getMessage());
   ex.printStackTrace(System.out);
   error.append(ex.getMessage());
   if(v!=null && !v.isEmpty())
   {
    v.removeAllElements();
    v = null;
   }
  }
  finally
  {
   dbcon.close();
   msg.delete(0,msg.length());
  }
  return v;
 }

} // end FetchData
