import javax.swing.table.*;
import java.util.*;

/**
 * SortedTableModel's class is copied from "Professional JAVA 
 * Programming" by Brett Spell.
 */

public class SortedTableModel extends AbstractTableModel
{
 protected TableModel sourceModel;
 protected int[] indexValues;
 protected TableSorter sorter;

 public SortedTableModel(TableModel model)
 {
  super();
  sourceModel = model;
 }
 public void renewModel(TableModel model)
 {
  sourceModel = null;
  sourceModel = model;
 }
 public int getRowCount()
 {
  return sourceModel.getRowCount();
 }
 public int getColumnCount()
 {
  return sourceModel.getColumnCount();
 }
 public Object getValueAt(int row,int column)
 {
  if(indexValues != null)
   row = getSourceIndex(row);
  return sourceModel.getValueAt(row,column);
 }
 public void setValueAt(Object value,int row,int column)
 {
  if(indexValues != null)
    row = getSourceIndex(row);
  sourceModel.setValueAt(value,row,column);
 }
 public boolean isCellEditable(int row,int column)
 {
  return sourceModel.isCellEditable(row,column);
 }
 public String getColumnName(int column)
 {
  return sourceModel.getColumnName(column); 
 }
 public Class getColumnClass(int column)
 {
  return sourceModel.getColumnClass(column);
 }
 public int getSourceIndex(int index)
 {
  if(indexValues != null)
   return indexValues[index];
  return -1;
 }

 public void clearSort()
 {
  indexValues = null;
  refreshViews();
 }
 public void refreshViews()
 {
  fireTableDataChanged();
 }

 public void sortByColumn(int column,int type,boolean isAscent) 
 {
  if(sorter == null) 
   sorter = new TableSorter(this);
  sorter.sort(column,type,isAscent);   
  fireTableDataChanged();
 }
  
 public int[] getIndexes() 
 {
  int n = getRowCount();
  if(indexValues != null) 
  {
   if(indexValues.length == n) 
    return indexValues;
  }
  indexValues = new int[n];
  for(int i=0; i<n; i++) 
   indexValues[i] = i;
  return indexValues;
 }
}
