import java.util.Vector;
import java.util.HashMap;
import java.util.StringTokenizer;

public class Translate
{
 public static final int ENG = 1;
 public static final int INA = 2;
 public static int usedLanguage = 0;

 public static final int ALLZERO = 0;
 public static final int ALLDIGIT = 1;
 public static final int LASTDIGIT= 2;
 public static final int MIDDIGIT = 3;
 public static final int FSTDIGIT = 4;
 public static final int LASTZERO = 5;
 public static final int MIDZERO  = 6;
 public static final int FSTZERO  = 7;
 public static final int ERROR = -1;

 public StringBuffer error = new StringBuffer(32);
 private HashMap englishTrans = new HashMap(24);
 private HashMap indonesiaTrans = new HashMap(22);
 
/**
 * Translate's empty constructor
 */
 public Translate()
 {
  this(Translate.INA); // default language
 }

/**
 * Translate's with parameter input
 */
 public Translate(int language)
 {
  determineLanguage(language);
  initialized();
 }

/**
 * isError's method is to see whether there is an error
 * @param none
 * @return true if error happens else return false
 */
 public boolean isError()
 {
  if(error.length()>0) return true;
  return false;
 }

/**
 * getError's method is to retrieve the error message
 * @param none
 * @return String of error message
 */
 public String getError()
 {
  return error.toString();
 }

/**
 * determineLanguage's method is to determine language to use
 * @param int i 
 * @return none
 */
 public void determineLanguage(int i)
 {
  if(i != Translate.ENG && i != Translate.INA)
   usedLanguage = 2; // default language
  else 
   usedLanguage = i;
 }

/**
 * translate's method is to actually translate the number into language.
 * It checks '.' for cent 
 * @param String input
 * @return String of the result
 */
 public String translate(String input)
 {
  if(input == null)
  {
   error.append("Input is null");
   return input;
  }
  if(input.length()<1) 
  {
   error.append("Input length is less than 1");
   return null;
  }
  String input2 = null;
  if(input.indexOf('.')>0)
  {
   int pos = input.indexOf('.');
   input2 = input.substring(pos+1,input.length());
   input = input.substring(0,pos); 
  }
  int length = input.length();
  StringBuffer result = new StringBuffer(32);
  switch(length)
  {
   case 1 : 
   case 2 :
   case 3 : result.append(processHundred(length,input)); break; 
   case 4 :
   case 5 :
   case 6 : result.append(processThousand(length,input)); break; 
   case 7 :
   case 8 :
   case 9 : result.append(processMillion(length,input)); break; 
   case 10:
   case 11:
   case 12: result.append(processBillion(length,input)); break; 
   default: error.append("No translation service available beyond 12 digits");
            return null;
  }
  if(isError()) return null;
  if(usedLanguage == INA)
  {
   result.append("RUPIAH");
   if(input2!=null && !input2.equalsIgnoreCase("00"))
    result.append(" DAN "+processHundred(input2.length(),input2)+"CENTS");
  }
  else
  {
   result.append("DOLLAR");
   if(input2!=null && !input2.equalsIgnoreCase("00"))
    result.append(" AND "+processHundred(input2.length(),input2)+"CENTS");
  }
  return result.toString();
 }

/**
 * processHundred is to process the number in term of hundred.
 * @param int len - contains length of input, String input contains the hundred
 * String input2 - contains the cent or null
 * @return String result
 */
 private String processHundred(int len,String input)
 {
  StringBuffer str = new StringBuffer(32);
  char array[] = new char[3];
  int start = 0;
  if(len == 1)
   start = 2;
  else if(len == 2)
   start = 1;
  input.getChars(0,input.length(),array,start);
  int type = checkPosition(array);
  if(type == ERROR)
  {
   error.append("Input to check exceed the max 3 digits\n");
   return "";
  }
  if(type == ALLZERO) return "";
  boolean belasFlag = false;
  if(usedLanguage == Translate.ENG)
  {
   if(type!=FSTZERO && type!=LASTDIGIT && type!=MIDDIGIT)
   {
    str.append((String)englishTrans.get(String.valueOf(array[0]))+" ");
    str.append((String)englishTrans.get("1xx")+" ");
   }
   if(type!=MIDZERO && type!=FSTDIGIT && type!=LASTDIGIT)
   {
    if(String.valueOf(array[1]).equalsIgnoreCase("1"))
    {
     str.append((String)englishTrans.get(String.valueOf(array[1])+
      String.valueOf(array[2]))+" ");
     belasFlag = true;
    }
    else
     str.append((String)englishTrans.get(String.valueOf(array[1])+"x")+" ");
   }
   if(type!=LASTZERO && type!=FSTDIGIT && type!=MIDDIGIT && !belasFlag)
    str.append((String)englishTrans.get(String.valueOf(array[2]))+" ");
  }
  else if(usedLanguage == Translate.INA)
  {
   if(String.valueOf(array[0]).equalsIgnoreCase("1"))
    str.append((String)indonesiaTrans.get("100")+" ");
   else if(type!=FSTZERO && type!=LASTDIGIT && type!=MIDDIGIT)
   {
    str.append((String)indonesiaTrans.get(String.valueOf(array[0]))+" ");
    str.append((String)indonesiaTrans.get("1xx")+" ");
   }
   if(type!=MIDZERO && type!=FSTDIGIT && type!=LASTDIGIT)
   {
    if(String.valueOf(array[1]).equalsIgnoreCase("1"))
    {
     str.append((String)indonesiaTrans.get(String.valueOf(array[1])+
      String.valueOf(array[2]))+" ");
     belasFlag = true;
    }
    else
    {
     str.append((String)indonesiaTrans.get(String.valueOf(array[1]))+" ");
     str.append((String)indonesiaTrans.get("1x")+" ");
    }
   }
   if(type!=LASTZERO && type!=FSTDIGIT && type!=MIDDIGIT && !belasFlag)
    str.append((String)indonesiaTrans.get(String.valueOf(array[2]))+" ");
  }
  else
  {
   error.append("No language service available\n");
   return null;
  }
  str.append("<BR>");
  return str.toString();
 }

/**
 * processThousand's method is to translate the thousand
 * @param int len - contain length of input, String input - contain input,
 * @return String of result 
 */
 private String processThousand(int len,String input)
 {
  StringBuffer str = new StringBuffer(32);
  String input1 = null; // for upper 
  String input2 = null; // for lower
  if(len == 6)
  {
   input1 = input.substring(0,3);
   input2 = input.substring(3,6);
  }
  else if(len == 5)
  {
   input1 = input.substring(0,2);
   input2 = input.substring(2,5);
  }
  else // len == 4
  {
   input1 = input.substring(0,1);
   input2 = input.substring(1,4);
  }
  boolean zeroflag = false;
  if(defineNonZero(input1) == ALLZERO)
   zeroflag = true;
  str.append(processHundred(input1.length(),input1));
  if(!zeroflag)
  {
   if(usedLanguage == Translate.ENG)
    str.append("THOUSAND ");
   else if(usedLanguage == Translate.INA)
    str.append("RIBU ");
   else
   {
    error.append("No service language available\n"); 
    return null;
   }
  }
  str.append(processHundred(input2.length(),input2));
  str.append("<BR>");
  return str.toString();
 }

/**
 * processMillion's method is to process million
 * @param int len - the length of input, String input - the input to process
 * @return String of result translation
 */
 private String processMillion(int len,String input)
 {
  StringBuffer str = new StringBuffer(32);
  String input1 = null; // for upper
  String input2 = null; // for middle
  if(len == 9)
  {
   input1 = input.substring(0,3);
   input2 = input.substring(3,9);
  }
  else if(len == 8)
  {
   input1 = input.substring(0,2);
   input2 = input.substring(2,8);
  }
  else // must be 7
  {
   input1 = input.substring(0,1);
   input2 = input.substring(1,7);
  }
  boolean zeroflag = false;
  if(defineNonZero(input1) == ALLZERO)
    zeroflag = true;
  str.append(processHundred(input1.length(),input1));
  if(!zeroflag)
  {
   if(usedLanguage == Translate.ENG)
    str.append("MILLION ");
   else if(usedLanguage == Translate.INA)
    str.append("JUTA ");
   else
   {
    error.append("No service language available\n"); 
    return null;
   }
  }
  str.append(processThousand(input2.length(),input2));
  str.append("<BR>");
  return str.toString();
 }

/**
 * processBillion's method is to process billion
 * @param int len - the length of input, String input - the input to process,
 * @return String of the result of translation
 */
 private String processBillion(int len,String input)
 {
  StringBuffer str = new StringBuffer(32);
  String input1 = null;
  String input2 = null;
  if(len == 12)
  {
   input1 = input.substring(0,3);
   input2 = input.substring(3,12);
  }
  else if(len == 11)
  {
   input1 = input.substring(0,2);
   input2 = input.substring(2,11);
  }
  else // must be 10
  {
   input1 = input.substring(0,1);
   input2 = input.substring(1,10);
  }
  str.append(processHundred(input1.length(),input1));
  if(usedLanguage == Translate.ENG)
   str.append("BILLION ");
  else if(usedLanguage == Translate.INA)
   str.append("MILIYAR ");
  else
  {
   error.append("No service language available\n"); 
   return null;
  }
  str.append(processMillion(input2.length(),input2));
  str.append("<BR>");
  return str.toString();
 }

/** 
 * defineNonZero's method is to define which is zero and non-zero in a string
 * @param String 
 * @return result of checkPosition
 */
 private int defineNonZero(String s)
 {
  char array[] = new char[3];
  int len = s.length();
  int start = 0;
  if(len == 1) start = 2;
  else if(len == 2) start = 1;
  s.getChars(0,s.length(),array,start);
  return checkPosition(array);
 }

/**
 * checkPosition's method is check all the position of the zero and non-zero
 * in the pool of 3 boolean and return the result in term of constant int
 * @param char[] - 3 characther long
 * @return the result else return ERROR
 */
 private int checkPosition(char[] s)
 {
  if(s.length>3) return ERROR;
  boolean pool[] = {false,false,false}; // assume all zeroes
  if(Character.getNumericValue(s[0])>0) pool[0] = true;
  if(Character.getNumericValue(s[1])>0) pool[1] = true;
  if(Character.getNumericValue(s[2])>0) pool[2] = true;
  if(pool[0]==false && pool[1]==false && pool[2]==false)
   return ALLZERO;
  if(pool[0]==false && pool[1]==false && pool[2]==true)
   return LASTDIGIT;
  if(pool[0]==false && pool[1]==true && pool[2]==false)
   return MIDDIGIT;
  if(pool[0]==true && pool[1]==false && pool[2]==false)
   return FSTDIGIT;
  if(pool[0]==true && pool[1]==true && pool[2]==false)
   return LASTZERO;
  if(pool[0]==true && pool[1]==false && pool[2]==true)
   return MIDZERO;
  if(pool[0]==false && pool[1]==true && pool[2]==true)
   return FSTZERO;
  return ALLDIGIT;
 }

/**
 * initialized's method is to initialize the english and indonesia translation
 * @param none
 * @return none
 */
 private void initialized()
 {
  initializeENG();
  initializeINA();
 }
 
/**
 * initializeENG's method is to initialize the translation 
 * from number to english
 * @param none
 * @return none
 */
 private void initializeENG()
 {
  englishTrans.put("0","ZERO");
  englishTrans.put("1","ONE");
  englishTrans.put("2","TWO");
  englishTrans.put("3","THREE");
  englishTrans.put("4","FOUR");
  englishTrans.put("5","FIVE");
  englishTrans.put("6","SIX");
  englishTrans.put("7","SEVEN");
  englishTrans.put("8","EIGHT");
  englishTrans.put("9","NINE");
  englishTrans.put("10","TEN");
  englishTrans.put("11","ELEVEN");
  englishTrans.put("12","TWELVE");
  englishTrans.put("13","THIRTEEN");
  englishTrans.put("14","FOURTEEN");
  englishTrans.put("15","FIFTEEN");
  englishTrans.put("16","SIXTEEN");
  englishTrans.put("17","SEVENTEEN");
  englishTrans.put("18","EIGHTEEN");
  englishTrans.put("19","NINETEEN");
  englishTrans.put("1xx","HUNDRED");
  englishTrans.put("1xxx","THOUSAND");
  englishTrans.put("1xxxx","MILLION");
  englishTrans.put("2x","TWENTY");
  englishTrans.put("3x","THIRTY");
  englishTrans.put("4x","FOURTY");
  englishTrans.put("5x","FIFTY");
  englishTrans.put("6x","SIXTY");
  englishTrans.put("7x","SEVENTY");
  englishTrans.put("8x","EIGHTY");
  englishTrans.put("9x","NINTY");
 }

/**
 * initializeINA's method is to initialize the translation 
 * from number to indonesia
 * @param none
 * @return none
 */
 private void initializeINA()
 {
  indonesiaTrans.put("0","NOL");
  indonesiaTrans.put("1","SATU");
  indonesiaTrans.put("2","DUA");
  indonesiaTrans.put("3","TIGA");
  indonesiaTrans.put("4","EMPAT");
  indonesiaTrans.put("5","LIMA");
  indonesiaTrans.put("6","ENAM");
  indonesiaTrans.put("7","TUJUH");
  indonesiaTrans.put("8","DELAPAN");
  indonesiaTrans.put("9","SEMBILAN");
  indonesiaTrans.put("10","SEPULUH");
  indonesiaTrans.put("11","SEBELAS");
  indonesiaTrans.put("12","DUA BELAS");
  indonesiaTrans.put("13","TIGA BELAS");
  indonesiaTrans.put("14","EMPAT BELAS");
  indonesiaTrans.put("15","LIMA BELAS");
  indonesiaTrans.put("16","ENAM BELAS");
  indonesiaTrans.put("17","TUJUH BELAS");
  indonesiaTrans.put("18","DELAPAN BELAS");
  indonesiaTrans.put("19","SEMBILAN BELAS");
  indonesiaTrans.put("100","SERATUS");
  indonesiaTrans.put("1000","SERIBU");
  indonesiaTrans.put("10000","SEJUTA");
  indonesiaTrans.put("100000","SEMILLIYAR");
  indonesiaTrans.put("1x","PULUH");
  indonesiaTrans.put("1xx","RATUS");
  indonesiaTrans.put("1xxx","RIBU");
  indonesiaTrans.put("1xxxx","JUTA");
  indonesiaTrans.put("1xxxxx","MILIYAR");
 }
}
